unit CheapMaterial;

interface

uses
  System.SysUtils, System.Classes, FMX.Types, FMX.Types3D, FMX.Materials, FMX.MaterialSources;

type
  TCheapMaterial = class(TCustomMaterial)
  protected
    procedure DoInitialize; override;
  end;

  TCheapMaterialSource = class(TMaterialSource)
  protected
    function CreateMaterial: TMaterial; override;
  end;

implementation

{ TCheapMaterial }

procedure TCheapMaterial.DoInitialize;

  function BytesFromFile(const FileName: string): TBytes;
  var
    ms: TMemoryStream;
  begin
    ms := TMemoryStream.Create;
    try
      ms.LoadFromFile(FileName);
      SetLength(Result, ms.Size);
      ms.ReadBuffer(Pointer(Result)^, ms.Size);
    finally
      ms.Free;
    end;
  end;

begin
  inherited;

  FVertexShader := TShaderManager.RegisterShaderFromData('cheap.vs', TContextShaderKind.skVertexShader, '', [
    TContextShaderSource.Create(TContextShaderArch.saDX9, BytesFromFile('cheapDX9.cvs'), [
      TContextShaderVariable.Create('MVPMatrix', TContextShaderVariableKind.vkMatrix, 0, 4)]
    ),
    TContextShaderSource.Create(TContextShaderArch.saDX10, BytesFromFile('cheapDX10.cvs'), [
      TContextShaderVariable.Create('MVPMatrix', TContextShaderVariableKind.vkMatrix, 0, 64)]
    )
  ]);
  FPixelShader := TShaderManager.RegisterShaderFromData('cheap.ps', TContextShaderKind.skPixelShader, '', [
    TContextShaderSource.Create(TContextShaderArch.saDX9, BytesFromFile('cheapDX9.cps'), []),
    TContextShaderSource.Create(TContextShaderArch.saDX10, BytesFromFile('cheapDX10.cps'), [])
  ]);
end;

{ TCheapMaterialSource }

function TCheapMaterialSource.CreateMaterial: TMaterial;
begin
  Result := TCheapMaterial.Create;
end;

initialization
  RegisterFmxClasses([TCheapMaterialSource]);
end.
